import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
import { WebComponentRenderer } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { ListPanel } from "./ListPanel";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { IgcListPanelTemplateItemUpdatingEventArgs } from './igc-list-panel-template-item-updating-event-args';
import { ListPanelOrientation_$type } from './ListPanelOrientation';
import { IgcListPanelContentRefreshedEventArgs } from './igc-list-panel-content-refreshed-event-args';
import { IgcListPanelTemplateWidthRequestedEventArgs } from './igc-list-panel-template-width-requested-event-args';
import { IgcListPanelTemplateHeightRequestedEventArgs } from './igc-list-panel-template-height-requested-event-args';
import { LocalDataSource } from "igniteui-webcomponents-core";
export let IgcListPanelComponent = /*@__PURE__*/ (() => {
    class IgcListPanelComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
            this.i.notifySizeChanged();
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
            this.i.notifySizeChanged();
        }
        get width() {
            return this._width;
        }
        createLocalDataSource(itemsSource) {
            let lds = new LocalDataSource();
            lds.dataSource = itemsSource;
            return lds;
        }
        onContentSizeChanged() {
        }
        constructor() {
            super();
            this._implementation = null;
            this._disconnected = false;
            this._dataSource = null;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._contentRefreshed = null;
            this._contentRefreshed_wrapped = null;
            this._rowUpdating = null;
            this._rowUpdating_wrapped = null;
            this._itemWidthRequested = null;
            this._itemWidthRequested_wrapped = null;
            this._itemHeightRequested = null;
            this._itemHeightRequested_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {});
            this._webComponentWrapper = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._webComponentWrapper);
            this._webComponentWrapper.setStyleProperty("display", "block");
            this._webComponentWrapper.setStyleProperty("width", "100%");
            this._webComponentWrapper.setStyleProperty("height", "100%");
            this.i.externalListPanel = this;
            this.i.provideContainer(this._webComponentRenderer);
            this.i.notifySizeChanged();
            this._webComponentRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        createImplementation() {
            return new ListPanel();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-list-panel");
            this.classList.add("igc-list-panel");
            var rootWrapper = this._webComponentRenderer.rootWrapper;
            var rootElement = rootWrapper.getNativeElement();
            this.appendChild(rootElement);
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            this.afterContentInit();
        }
        afterContentInit() {
            this.i.notifySizeChanged();
        }
        set dataSource(value) {
            this._dataSource = value;
            if (this._implementation != null) {
                this._implementation.itemsSource = this._dataSource;
            }
        }
        get dataSource() {
            return this._dataSource;
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcListPanelComponent._observedAttributesIgcListPanelComponent == null) {
                let names = getAllPropertyNames(IgcListPanelComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcListPanelComponent._observedAttributesIgcListPanelComponent = names;
            }
            return IgcListPanelComponent._observedAttributesIgcListPanelComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcListPanelComponent._isElementRegistered) {
                IgcListPanelComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcListPanelComponent.htmlTagName, IgcListPanelComponent);
            }
        }
        /**
     * Gets or Sets the property name that contains the values.
    */
        get rowHeight() {
            return this.i.a7;
        }
        set rowHeight(v) {
            this.i.a7 = +v;
            this._a("rowHeight", this.i.a7);
        }
        get actualRowHeight() {
            return this.i.av;
        }
        set actualRowHeight(v) {
            this.i.av = +v;
            this._a("actualRowHeight", this.i.av);
        }
        /**
         * Gets or Sets the property name that contains the values.
        */
        get cellTextStyle() {
            if (this.i.s == null) {
                return null;
            }
            return this.i.s.fontString;
        }
        set cellTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.s = fi;
            this._a("cellTextStyle", this.i.s != null ? this.i.s.fontString : "");
        }
        get hasUnevenSizes() {
            return this.i.ah;
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get backgroundColor() {
            return brushToString(this.i.cs);
        }
        set backgroundColor(v) {
            this.i.cs = stringToBrush(v);
            this._a("backgroundColor", brushToString(this.i.cs));
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get orientation() {
            return this.i.k;
        }
        set orientation(v) {
            this.i.k = ensureEnum(ListPanelOrientation_$type, v);
            this._a("orientation", enumToString(ListPanelOrientation_$type, this.i.k));
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get isCustomRowHeightEnabled() {
            return this.i.ai;
        }
        set isCustomRowHeightEnabled(v) {
            this.i.ai = ensureBool(v);
            this._a("isCustomRowHeightEnabled", this.i.ai);
        }
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get textColor() {
            return brushToString(this.i.cv);
        }
        set textColor(v) {
            this.i.cv = stringToBrush(v);
            this._a("textColor", brushToString(this.i.cv));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ListPanelComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        /**
     * Gets the last visible index in the grid.
    
    */
        getLastVisibleIndex() {
            let iv = this.i.a4();
            return (iv);
        }
        /**
         * Gets the first visible index in the grid.
        
        */
        getFirstVisibleIndex() {
            let iv = this.i.a3();
            return (iv);
        }
        /**
         * Invalidates the bound data for the rows currently visible.
        
        */
        invalidateVisibleItems() {
            this.i.by();
        }
        notifySetItem(index, oldItem, newItem) {
            this.i.b4(index, oldItem, newItem);
        }
        /**
         * Manually notifies the grid's data source that the data it has bound to has been cleared and needs to be re-examined.
         * This should not be called if the data that the grid is bound to is already observable.
        
        */
        notifyClearItems() {
            this.i.b1();
        }
        notifyInsertItem(index, newItem) {
            this.i.b2(index, newItem);
        }
        notifyRemoveItem(index, oldItem) {
            this.i.b3(index, oldItem);
        }
        /**
         * Returns the data index of an item within the bound data source.
        
        * @param item  * The item from which to get the index.
        */
        dataIndexOfItem(item) {
            let iv = this.i.a1(item);
            return (iv);
        }
        dataIndexOfPrimaryKey(key) {
            let iv = this.i.a2(key);
            return (iv);
        }
        get contentRefreshed() {
            return this._contentRefreshed;
        }
        set contentRefreshed(ev) {
            if (this._contentRefreshed_wrapped !== null) {
                this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
                this._contentRefreshed_wrapped = null;
                this._contentRefreshed = null;
            }
            this._contentRefreshed = ev;
            this._contentRefreshed_wrapped = (o, e) => {
                let outerArgs = new IgcListPanelContentRefreshedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeContentRefreshed) {
                    this.beforeContentRefreshed(this, outerArgs);
                }
                if (this._contentRefreshed) {
                    this._contentRefreshed(this, outerArgs);
                }
            };
            this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            ;
        }
        get rowUpdating() {
            return this._rowUpdating;
        }
        set rowUpdating(ev) {
            if (this._rowUpdating_wrapped !== null) {
                this.i.rowUpdating = delegateRemove(this.i.rowUpdating, this._rowUpdating_wrapped);
                this._rowUpdating_wrapped = null;
                this._rowUpdating = null;
            }
            this._rowUpdating = ev;
            this._rowUpdating_wrapped = (o, e) => {
                let outerArgs = new IgcListPanelTemplateItemUpdatingEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeRowUpdating) {
                    this.beforeRowUpdating(this, outerArgs);
                }
                if (this._rowUpdating) {
                    this._rowUpdating(this, outerArgs);
                }
            };
            this.i.rowUpdating = delegateCombine(this.i.rowUpdating, this._rowUpdating_wrapped);
            ;
        }
        get itemWidthRequested() {
            return this._itemWidthRequested;
        }
        set itemWidthRequested(ev) {
            if (this._itemWidthRequested_wrapped !== null) {
                this.i.itemWidthRequested = delegateRemove(this.i.itemWidthRequested, this._itemWidthRequested_wrapped);
                this._itemWidthRequested_wrapped = null;
                this._itemWidthRequested = null;
            }
            this._itemWidthRequested = ev;
            this._itemWidthRequested_wrapped = (o, e) => {
                let outerArgs = new IgcListPanelTemplateWidthRequestedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeItemWidthRequested) {
                    this.beforeItemWidthRequested(this, outerArgs);
                }
                if (this._itemWidthRequested) {
                    this._itemWidthRequested(this, outerArgs);
                }
            };
            this.i.itemWidthRequested = delegateCombine(this.i.itemWidthRequested, this._itemWidthRequested_wrapped);
            ;
        }
        get itemHeightRequested() {
            return this._itemHeightRequested;
        }
        set itemHeightRequested(ev) {
            if (this._itemHeightRequested_wrapped !== null) {
                this.i.itemHeightRequested = delegateRemove(this.i.itemHeightRequested, this._itemHeightRequested_wrapped);
                this._itemHeightRequested_wrapped = null;
                this._itemHeightRequested = null;
            }
            this._itemHeightRequested = ev;
            this._itemHeightRequested_wrapped = (o, e) => {
                let outerArgs = new IgcListPanelTemplateHeightRequestedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeItemHeightRequested) {
                    this.beforeItemHeightRequested(this, outerArgs);
                }
                if (this._itemHeightRequested) {
                    this._itemHeightRequested(this, outerArgs);
                }
            };
            this.i.itemHeightRequested = delegateCombine(this.i.itemHeightRequested, this._itemHeightRequested_wrapped);
            ;
        }
    }
    IgcListPanelComponent._observedAttributesIgcListPanelComponent = null;
    IgcListPanelComponent.htmlTagName = "igc-list-panel";
    IgcListPanelComponent._isElementRegistered = false;
    return IgcListPanelComponent;
})();
