import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { Visibility_$type } from "igniteui-webcomponents-core";
import { IgcToolCommandEventArgs } from "./igc-tool-command-event-args";
import { IgcToolActionPerformedEventArgs } from "./igc-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-webcomponents-core";
import { NamePatcher, getAllPropertyNames, fromSpinal, toSpinal, brushToString, stringToBrush, ensureBool, ensureEnum, enumToString, initializePropertiesFromCss } from "igniteui-webcomponents-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export let IgcToolActionComponent = /*@__PURE__*/ (() => {
    class IgcToolActionComponent extends IgcHTMLElement {
        createImplementation() {
            return null;
        }
        /**
         * @hidden
         */
        get i() {
            return this._implementation;
        } /**
                                     * @hidden
                                     */
        static _createFromInternal(internal) {
            if (!internal) {
                return null;
            }
            if (!internal.$type) {
                return null;
            }
            let name = internal.$type.name;
            let externalName = "Igc" + name + "Component";
            if (!TypeRegistrar.isRegistered(externalName)) {
                return null;
            }
            return TypeRegistrar.create(externalName);
        }
        onImplementationCreated() {
        }
        constructor() {
            super();
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this._actions = null;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            this._performed = null;
            this._performed_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this.onImplementationCreated();
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        connectedCallback() {
            if (super["connectedCallback"]) {
                super["connectedCallback"]();
            }
            if (this.i.connectedCallback) {
                this.i.connectedCallback();
            }
            if (!this._attached) {
                this._attached = true;
                this._flushQueuedAttributes();
            }
        }
        disconnectedCallback() {
            if (super["disconnectedCallback"]) {
                super["disconnectedCallback"]();
            }
            if (this.i.disconnectedCallback) {
                this.i.disconnectedCallback();
            }
            if (this._attached) {
                this._attached = false;
            }
        }
        static get observedAttributes() {
            if (IgcToolActionComponent._observedAttributesIgcToolActionComponent == null) {
                let names = getAllPropertyNames(IgcToolActionComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolActionComponent._observedAttributesIgcToolActionComponent = names;
            }
            return IgcToolActionComponent._observedAttributesIgcToolActionComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        get id() {
            return this.i.cs;
        }
        /**
         * Gets the sub actions associated with this tool action.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let innerColl = this.i.actions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actions = coll._fromInner(innerColl);
            }
            return this._actions;
        }
        set actions(v) {
            if (this._actions !== null) {
                this._actions._setSyncTarget(null);
                this._actions = null;
            }
            let coll = new IgcToolActionCollection();
            this._actions = coll._fromOuter(v);
            let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            let innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actions._setSyncTarget(syncColl);
        }
        /**
         * Gets or sets the width to use for this tool action.
        */
        get width() {
            return this.i.bi;
        }
        set width(v) {
            this.i.bi = +v;
            this._a("width", this.i.bi);
        }
        get parentId() {
            return this.i.c1;
        }
        set parentId(v) {
            this.i.c1 = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted before.
         * Subsequent actions will continue to insert before the target action, following
         * the original action that set this property. If using this property do not use the
         * AfterId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get beforeId() {
            return this.i.b9;
        }
        set beforeId(v) {
            this.i.b9 = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted after.
         * Subsequent actions will continue to insert after the target action, following
         * the original action that set this property. If using this property do not use the
         * BeforeId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get afterId() {
            return this.i.b6;
        }
        set afterId(v) {
            this.i.b6 = v;
        }
        /**
         * Sets to the Id of the action that this action will overlay itself on to.
         * The purpose of this property is to allow users to override actions automatically generated by
         * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
         * with them that can be used for the OverlayId.
         * These overlaid actions will
         * not
         * replace the generated one but will have its properties
         * overlaid onto the generated actions.
        */
        get overlayId() {
            return this.i.cw;
        }
        set overlayId(v) {
            this.i.cw = v;
        }
        get commandId() {
            return this.i.cd;
        }
        set commandId(v) {
            this.i.cd = v;
        }
        get commandArgument() {
            return this.i.ca;
        }
        set commandArgument(v) {
            this.i.ca = v;
        }
        get commandArgumentValue() {
            return this.i.bk;
        }
        set commandArgumentValue(v) {
            this.i.bk = v;
        }
        get paddingLeft() {
            return this.i.be;
        }
        set paddingLeft(v) {
            this.i.be = +v;
            this._a("paddingLeft", this.i.be);
        }
        get actualPaddingLeft() {
            return this.i.a2;
        }
        set actualPaddingLeft(v) {
            this.i.a2 = +v;
            this._a("actualPaddingLeft", this.i.a2);
        }
        get paddingRight() {
            return this.i.bf;
        }
        set paddingRight(v) {
            this.i.bf = +v;
            this._a("paddingRight", this.i.bf);
        }
        get actualPaddingRight() {
            return this.i.a3;
        }
        set actualPaddingRight(v) {
            this.i.a3 = +v;
            this._a("actualPaddingRight", this.i.a3);
        }
        get paddingBottom() {
            return this.i.bd;
        }
        set paddingBottom(v) {
            this.i.bd = +v;
            this._a("paddingBottom", this.i.bd);
        }
        get actualPaddingBottom() {
            return this.i.a1;
        }
        set actualPaddingBottom(v) {
            this.i.a1 = +v;
            this._a("actualPaddingBottom", this.i.a1);
        }
        get paddingTop() {
            return this.i.bg;
        }
        set paddingTop(v) {
            this.i.bg = +v;
            this._a("paddingTop", this.i.bg);
        }
        get actualPaddingTop() {
            return this.i.a4;
        }
        set actualPaddingTop(v) {
            this.i.a4 = +v;
            this._a("actualPaddingTop", this.i.a4);
        }
        /**
         * Gets the actual hover background used by the tool action.
        */
        get actualHoverBackground() {
            return brushToString(this.i.eq);
        }
        set actualHoverBackground(v) {
            this.i.eq = stringToBrush(v);
            this._a("actualHoverBackground", brushToString(this.i.eq));
        }
        /**
         * Gets or sets the hover background for this tool action.
        */
        get hoverBackground() {
            return brushToString(this.i.ew);
        }
        set hoverBackground(v) {
            this.i.ew = stringToBrush(v);
            this._a("hoverBackground", brushToString(this.i.ew));
        }
        /**
         * Gets the actual background used by the tool action.
        */
        get actualBackground() {
            return brushToString(this.i.eo);
        }
        set actualBackground(v) {
            this.i.eo = stringToBrush(v);
            this._a("actualBackground", brushToString(this.i.eo));
        }
        /**
         * Gets or sets the background color of the tool action.
        */
        get background() {
            return brushToString(this.i.eu);
        }
        set background(v) {
            this.i.eu = stringToBrush(v);
            this._a("background", brushToString(this.i.eu));
        }
        /**
         * Gets the actual label text color.
        */
        get actualTextColor() {
            return brushToString(this.i.et);
        }
        set actualTextColor(v) {
            this.i.et = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.et));
        }
        /**
         * Gets or sets the label text color.
        */
        get textColor() {
            return brushToString(this.i.ez);
        }
        set textColor(v) {
            this.i.ez = stringToBrush(v);
            this._a("textColor", brushToString(this.i.ez));
        }
        /**
         * Gets the actual label text color when the tool is disabled.
        */
        get actualDisabledTextColor() {
            return brushToString(this.i.ep);
        }
        set actualDisabledTextColor(v) {
            this.i.ep = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.ep));
        }
        /**
         * Gets or sets the label text color when the tool is disabled.
        */
        get disabledTextColor() {
            return brushToString(this.i.ev);
        }
        set disabledTextColor(v) {
            this.i.ev = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.ev));
        }
        /**
         * Gets the actual font used.
        */
        get actualTextStyle() {
            if (this.i.r == null) {
                return null;
            }
            return this.i.r.fontString;
        }
        set actualTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.r = fi;
            this._a("actualTextStyle", this.i.r != null ? this.i.r.fontString : "");
        }
        /**
         * Gets or sets the font to use.
        */
        get textStyle() {
            if (this.i.t == null) {
                return null;
            }
            return this.i.t.fontString;
        }
        set textStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.t = fi;
            this._a("textStyle", this.i.t != null ? this.i.t.fontString : "");
        }
        get isDisabled() {
            return this.i.ah;
        }
        set isDisabled(v) {
            this.i.ah = ensureBool(v);
            this._a("isDisabled", this.i.ah);
        }
        get name() {
            return this.i.cu;
        }
        set name(v) {
            this.i.cu = v;
        }
        /**
         * Gets or sets the title of the tool.
        */
        get title() {
            return this.i.da;
        }
        set title(v) {
            this.i.da = v;
        }
        /**
         * Gets or sets the subtitle of the tool.
        */
        get subtitle() {
            return this.i.c6;
        }
        set subtitle(v) {
            this.i.c6 = v;
        }
        get iconName() {
            return this.i.cl;
        }
        set iconName(v) {
            this.i.cl = v;
        }
        get iconCollectionName() {
            return this.i.ci;
        }
        set iconCollectionName(v) {
            this.i.ci = v;
        }
        /**
         * Gets or sets the icon viewbox left.
        */
        get iconViewBoxLeft() {
            return this.i.ba;
        }
        set iconViewBoxLeft(v) {
            this.i.ba = +v;
            this._a("iconViewBoxLeft", this.i.ba);
        }
        /**
         * Gets or sets the icon viewbox top.
        */
        get iconViewBoxTop() {
            return this.i.bb;
        }
        set iconViewBoxTop(v) {
            this.i.bb = +v;
            this._a("iconViewBoxTop", this.i.bb);
        }
        /**
         * Gets or sets the icon viewbox width.
        */
        get iconViewBoxWidth() {
            return this.i.bc;
        }
        set iconViewBoxWidth(v) {
            this.i.bc = +v;
            this._a("iconViewBoxWidth", this.i.bc);
        }
        /**
         * Gets or sets the icon viewbox height.
        */
        get iconViewBoxHeight() {
            return this.i.a9;
        }
        set iconViewBoxHeight(v) {
            this.i.a9 = +v;
            this._a("iconViewBoxHeight", this.i.a9);
        }
        /**
         * Gets or sets the icon fill color.
        */
        get iconFill() {
            return brushToString(this.i.ex);
        }
        set iconFill(v) {
            this.i.ex = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.ex));
        }
        /**
         * Gets the actual icon fill color.
        */
        get actualIconFill() {
            return brushToString(this.i.er);
        }
        set actualIconFill(v) {
            this.i.er = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.er));
        }
        /**
         * Gets or sets the icon stroke color.
        */
        get iconStroke() {
            return brushToString(this.i.ey);
        }
        set iconStroke(v) {
            this.i.ey = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.ey));
        }
        /**
         * Gets the actual icon stroke color.
        */
        get actualIconStroke() {
            return brushToString(this.i.es);
        }
        set actualIconStroke(v) {
            this.i.es = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.es));
        }
        /**
         * Gets or sets the tool action visibility.
        */
        get visibility() {
            return this.i.e1;
        }
        set visibility(v) {
            this.i.e1 = ensureEnum(Visibility_$type, v);
            this._a("visibility", enumToString(Visibility_$type, this.i.e1));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolActionComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        cleanUp() {
            this.i.dd();
        }
        fromInfo(info) {
            this.i.dp(info);
        }
        openSubMenu() {
            this.i.d2();
        }
        closeSubmenu() {
            this.i.df();
        }
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
        get performed() {
            return this._performed;
        }
        set performed(ev) {
            if (this._performed_wrapped !== null) {
                this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
                this._performed_wrapped = null;
                this._performed = null;
            }
            this._performed = ev;
            this._performed_wrapped = (o, e) => {
                let outerArgs = new IgcToolActionPerformedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforePerformed) {
                    this.beforePerformed(this, outerArgs);
                }
                if (this._performed) {
                    this._performed(this, outerArgs);
                }
            };
            this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
            ;
        }
    }
    IgcToolActionComponent._observedAttributesIgcToolActionComponent = null;
    return IgcToolActionComponent;
})();
