import { fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
import { WebComponentRenderer, PortalManager } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { CollectionAdapter } from "igniteui-webcomponents-core";
import { NotifyCollectionChangedAction } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { ToolPanel } from "./ToolPanel";
import { BaseControlTheme_$type } from "igniteui-webcomponents-core";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { ToolPanelOrientation_$type } from './ToolPanelOrientation';
import { IgcToolActionComponent } from "./igc-tool-action-component";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { IgcToolPanelContentRefreshedEventArgs } from "./igc-tool-panel-content-refreshed-event-args";
import { IgcToolCommandEventArgs } from './igc-tool-command-event-args';
import { NestedActionMode_$type } from './NestedActionMode';
export let IgcToolPanelComponent = /*@__PURE__*/ (() => {
    class IgcToolPanelComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
            this.i.notifySizeChanged();
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
            this.i.notifySizeChanged();
        }
        get width() {
            return this._width;
        }
        /**
         * A collection of manually added actions for the editor.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let inner = coll._innerColl;
                inner.addListener((sender, e) => {
                    switch (e.action) {
                        case NotifyCollectionChangedAction.Add:
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            break;
                        case NotifyCollectionChangedAction.Replace:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            this._actionsAdapter.clearManualItems();
                            break;
                    }
                });
                this._actions = coll;
            }
            return this._actions;
        }
        constructor() {
            super();
            this._implementation = null;
            this.contentActions = [];
            /**
            * The actions actually present in the editor. Do not directly modify this array.
            * This array's contents can be modified by using the actions property or providing child content.
            */
            this.actualActions = [];
            this._actions = null;
            this._actionsAdapter = null;
            this._disconnected = false;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._contentRefreshed = null;
            this._contentRefreshed_wrapped = null;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this._portalManager = new PortalManager("toolPanelContent");
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {}, this._portalManager);
            this._webComponentWrapper = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._webComponentWrapper);
            this._webComponentWrapper.setStyleProperty("display", "block");
            this._webComponentWrapper.setStyleProperty("width", "100%");
            this._webComponentWrapper.setStyleProperty("height", "100%");
            this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.actualActions, (c) => c.i, (i) => {
                if (this._webComponentRenderer && this._webComponentRenderer.rootWrapper.getNativeElement().parentElement) {
                    i._styling(this, this, this);
                }
            }, (i) => { });
            this.i.provideContainer(this._webComponentRenderer);
            this.i.notifySizeChanged();
            let mut = new MutationObserver((list) => {
                for (var mutation of list) {
                    if (mutation.type == 'childList') {
                        this.updateContentProperties();
                    }
                }
            });
            mut.observe(this, {
                childList: true
            });
            this._webComponentRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        updateContentProperties() {
            this.contentActions.length = 0;
            for (var i = 0; i < this.children.length; i++) {
                if (this.children[i] instanceof IgcToolActionComponent) {
                    this.contentActions.push(this.children[i]);
                }
            }
            if (this._actionsAdapter != null) {
                this._actionsAdapter.notifyContentChanged();
            }
        }
        createImplementation() {
            return new ToolPanel();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-tool-panel");
            this.classList.add("igc-tool-panel");
            var rootWrapper = this._webComponentRenderer.rootWrapper;
            var rootElement = rootWrapper.getNativeElement();
            this.appendChild(rootElement);
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            for (let i = 0; i < this.actualActions.length; i++) {
                let s = this.actualActions[i];
                s._styling(this, this, this);
            }
            this.afterContentInit();
        }
        afterContentInit() {
            this.i.notifySizeChanged();
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcToolPanelComponent._observedAttributesIgcToolPanelComponent == null) {
                let names = getAllPropertyNames(IgcToolPanelComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolPanelComponent._observedAttributesIgcToolPanelComponent = names;
            }
            return IgcToolPanelComponent._observedAttributesIgcToolPanelComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcToolPanelComponent._isElementRegistered) {
                IgcToolPanelComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcToolPanelComponent.htmlTagName, IgcToolPanelComponent);
            }
        }
        /**
     * Gets or sets if the tool actions should display their submenus on mouse hover.
    */
        get showOnHover() {
            return this.i.ac;
        }
        set showOnHover(v) {
            this.i.ac = ensureBool(v);
            this._a("showOnHover", this.i.ac);
        }
        /**
         * Gets or Sets the property name that contains the values.
        */
        get cellTextStyle() {
            if (this.i.t == null) {
                return null;
            }
            return this.i.t.fontString;
        }
        set cellTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.t = fi;
            this._a("cellTextStyle", this.i.t != null ? this.i.t.fontString : "");
        }
        /**
         * Gets or sets the base built in theme to use for the checkbox.
        */
        get baseTheme() {
            return this.i.r;
        }
        set baseTheme(v) {
            this.i.r = ensureEnum(BaseControlTheme_$type, v);
            this._a("baseTheme", enumToString(BaseControlTheme_$type, this.i.r));
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get backgroundColor() {
            return brushToString(this.i.e7);
        }
        set backgroundColor(v) {
            this.i.e7 = stringToBrush(v);
            this._a("backgroundColor", brushToString(this.i.e7));
        }
        get actualBackgroundColor() {
            return brushToString(this.i.eu);
        }
        set actualBackgroundColor(v) {
            this.i.eu = stringToBrush(v);
            this._a("actualBackgroundColor", brushToString(this.i.eu));
        }
        /**
         * Gets or sets the background color for submenus.
        */
        get submenuBackgroundColor() {
            return brushToString(this.i.fg);
        }
        set submenuBackgroundColor(v) {
            this.i.fg = stringToBrush(v);
            this._a("submenuBackgroundColor", brushToString(this.i.fg));
        }
        get actualSubmenuBackgroundColor() {
            return brushToString(this.i.e3);
        }
        set actualSubmenuBackgroundColor(v) {
            this.i.e3 = stringToBrush(v);
            this._a("actualSubmenuBackgroundColor", brushToString(this.i.e3));
        }
        /**
         * Gets or sets the group header background color.
        */
        get groupHeaderBackgroundColor() {
            return brushToString(this.i.e9);
        }
        set groupHeaderBackgroundColor(v) {
            this.i.e9 = stringToBrush(v);
            this._a("groupHeaderBackgroundColor", brushToString(this.i.e9));
        }
        get actualGroupHeaderBackgroundColor() {
            return brushToString(this.i.ew);
        }
        set actualGroupHeaderBackgroundColor(v) {
            this.i.ew = stringToBrush(v);
            this._a("actualGroupHeaderBackgroundColor", brushToString(this.i.ew));
        }
        /**
         * Gets or sets the separator background color.
        */
        get separatorBackgroundColor() {
            return brushToString(this.i.ff);
        }
        set separatorBackgroundColor(v) {
            this.i.ff = stringToBrush(v);
            this._a("separatorBackgroundColor", brushToString(this.i.ff));
        }
        get actualSeparatorBackgroundColor() {
            return brushToString(this.i.e2);
        }
        set actualSeparatorBackgroundColor(v) {
            this.i.e2 = stringToBrush(v);
            this._a("actualSeparatorBackgroundColor", brushToString(this.i.e2));
        }
        /**
         * Gets or sets the group header separator background color.
        */
        get groupHeaderSeparatorBackgroundColor() {
            return brushToString(this.i.fa);
        }
        set groupHeaderSeparatorBackgroundColor(v) {
            this.i.fa = stringToBrush(v);
            this._a("groupHeaderSeparatorBackgroundColor", brushToString(this.i.fa));
        }
        get actualGroupHeaderSeparatorBackgroundColor() {
            return brushToString(this.i.ex);
        }
        set actualGroupHeaderSeparatorBackgroundColor(v) {
            this.i.ex = stringToBrush(v);
            this._a("actualGroupHeaderSeparatorBackgroundColor", brushToString(this.i.ex));
        }
        /**
         * Gets or sets the hover background color.
        */
        get hoverBackgroundColor() {
            return brushToString(this.i.fc);
        }
        set hoverBackgroundColor(v) {
            this.i.fc = stringToBrush(v);
            this._a("hoverBackgroundColor", brushToString(this.i.fc));
        }
        get actualHoverBackgroundColor() {
            return brushToString(this.i.ez);
        }
        set actualHoverBackgroundColor(v) {
            this.i.ez = stringToBrush(v);
            this._a("actualHoverBackgroundColor", brushToString(this.i.ez));
        }
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get textColor() {
            return brushToString(this.i.fh);
        }
        set textColor(v) {
            this.i.fh = stringToBrush(v);
            this._a("textColor", brushToString(this.i.fh));
        }
        get actualTextColor() {
            return brushToString(this.i.e4);
        }
        set actualTextColor(v) {
            this.i.e4 = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.e4));
        }
        /**
         * Gets or sets the color to use for the text in disabled tool actions.
        */
        get disabledTextColor() {
            return brushToString(this.i.e8);
        }
        set disabledTextColor(v) {
            this.i.e8 = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.e8));
        }
        get actualDisabledTextColor() {
            return brushToString(this.i.ev);
        }
        set actualDisabledTextColor(v) {
            this.i.ev = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.ev));
        }
        /**
         * Gets or sets the text color for group header tool actions.
        */
        get groupHeaderTextColor() {
            return brushToString(this.i.fb);
        }
        set groupHeaderTextColor(v) {
            this.i.fb = stringToBrush(v);
            this._a("groupHeaderTextColor", brushToString(this.i.fb));
        }
        get actualGroupHeaderTextColor() {
            return brushToString(this.i.ey);
        }
        set actualGroupHeaderTextColor(v) {
            this.i.ey = stringToBrush(v);
            this._a("actualGroupHeaderTextColor", brushToString(this.i.ey));
        }
        /**
         * Gets or sets the tooltip text color.
        */
        get toolTipTextColor() {
            return brushToString(this.i.fj);
        }
        set toolTipTextColor(v) {
            this.i.fj = stringToBrush(v);
            this._a("toolTipTextColor", brushToString(this.i.fj));
        }
        get actualToolTipTextColor() {
            return brushToString(this.i.e6);
        }
        set actualToolTipTextColor(v) {
            this.i.e6 = stringToBrush(v);
            this._a("actualToolTipTextColor", brushToString(this.i.e6));
        }
        /**
         * Gets or sets the tooltip background color.
        */
        get toolTipBackgroundColor() {
            return brushToString(this.i.fi);
        }
        set toolTipBackgroundColor(v) {
            this.i.fi = stringToBrush(v);
            this._a("toolTipBackgroundColor", brushToString(this.i.fi));
        }
        /**
         * Gets the actual tooltip background color.
        */
        get actualToolTipBackgroundColor() {
            return brushToString(this.i.e5);
        }
        set actualToolTipBackgroundColor(v) {
            this.i.e5 = stringToBrush(v);
            this._a("actualToolTipBackgroundColor", brushToString(this.i.e5));
        }
        /**
         * Gets or sets the tooltip corner radius.
        */
        get toolTipCornerRadius() {
            return this.i.bl;
        }
        set toolTipCornerRadius(v) {
            this.i.bl = +v;
            this._a("toolTipCornerRadius", this.i.bl);
        }
        /**
         * Gets the actual tooltip corner radius.
        */
        get actualToolTipCornerRadius() {
            return this.i.bb;
        }
        set actualToolTipCornerRadius(v) {
            this.i.bb = +v;
            this._a("actualToolTipCornerRadius", this.i.bb);
        }
        /**
         * Gets or sets the shadow elevation for the tooltip.
        */
        get toolTipElevation() {
            return this.i.bm;
        }
        set toolTipElevation(v) {
            this.i.bm = +v;
            this._a("toolTipElevation", this.i.bm);
        }
        /**
         * Gets the actual shadow elevation used by the tooltip.
        */
        get actualToolTipElevation() {
            return this.i.bc;
        }
        set actualToolTipElevation(v) {
            this.i.bc = +v;
            this._a("actualToolTipElevation", this.i.bc);
        }
        /**
         * Gets or sets the fill color of icons in the ToolPanel.
        */
        get iconFill() {
            return brushToString(this.i.fd);
        }
        set iconFill(v) {
            this.i.fd = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.fd));
        }
        /**
         * Gets the actual icon fill color.
        */
        get actualIconFill() {
            return brushToString(this.i.e0);
        }
        set actualIconFill(v) {
            this.i.e0 = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.e0));
        }
        /**
         * Gets or sets the stroke color of icons in the ToolPanel.
        */
        get iconStroke() {
            return brushToString(this.i.fe);
        }
        set iconStroke(v) {
            this.i.fe = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.fe));
        }
        /**
         * Gets the actual icon stroke color.
        */
        get actualIconStroke() {
            return brushToString(this.i.e1);
        }
        set actualIconStroke(v) {
            this.i.e1 = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.e1));
        }
        /**
         * Gets or sets the left padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingLeft() {
            return this.i.bi;
        }
        set separatorVerticalPaddingLeft(v) {
            this.i.bi = +v;
            this._a("separatorVerticalPaddingLeft", this.i.bi);
        }
        get actualSeparatorVerticalPaddingLeft() {
            return this.i.a8;
        }
        set actualSeparatorVerticalPaddingLeft(v) {
            this.i.a8 = +v;
            this._a("actualSeparatorVerticalPaddingLeft", this.i.a8);
        }
        /**
         * Gets or sets the right padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingRight() {
            return this.i.bj;
        }
        set separatorVerticalPaddingRight(v) {
            this.i.bj = +v;
            this._a("separatorVerticalPaddingRight", this.i.bj);
        }
        get actualSeparatorVerticalPaddingRight() {
            return this.i.a9;
        }
        set actualSeparatorVerticalPaddingRight(v) {
            this.i.a9 = +v;
            this._a("actualSeparatorVerticalPaddingRight", this.i.a9);
        }
        /**
         * Gets or sets the top padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingTop() {
            return this.i.bk;
        }
        set separatorVerticalPaddingTop(v) {
            this.i.bk = +v;
            this._a("separatorVerticalPaddingTop", this.i.bk);
        }
        get actualSeparatorVerticalPaddingTop() {
            return this.i.ba;
        }
        set actualSeparatorVerticalPaddingTop(v) {
            this.i.ba = +v;
            this._a("actualSeparatorVerticalPaddingTop", this.i.ba);
        }
        /**
         * Gets or sets the bottom padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingBottom() {
            return this.i.bh;
        }
        set separatorVerticalPaddingBottom(v) {
            this.i.bh = +v;
            this._a("separatorVerticalPaddingBottom", this.i.bh);
        }
        get actualSeparatorVerticalPaddingBottom() {
            return this.i.a7;
        }
        set actualSeparatorVerticalPaddingBottom(v) {
            this.i.a7 = +v;
            this._a("actualSeparatorVerticalPaddingBottom", this.i.a7);
        }
        /**
         * Gets or sets the left padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingLeft() {
            return this.i.be;
        }
        set separatorHorizontalPaddingLeft(v) {
            this.i.be = +v;
            this._a("separatorHorizontalPaddingLeft", this.i.be);
        }
        get actualSeparatorHorizontalPaddingLeft() {
            return this.i.a4;
        }
        set actualSeparatorHorizontalPaddingLeft(v) {
            this.i.a4 = +v;
            this._a("actualSeparatorHorizontalPaddingLeft", this.i.a4);
        }
        /**
         * Gets or sets the right padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingRight() {
            return this.i.bf;
        }
        set separatorHorizontalPaddingRight(v) {
            this.i.bf = +v;
            this._a("separatorHorizontalPaddingRight", this.i.bf);
        }
        get actualSeparatorHorizontalPaddingRight() {
            return this.i.a5;
        }
        set actualSeparatorHorizontalPaddingRight(v) {
            this.i.a5 = +v;
            this._a("actualSeparatorHorizontalPaddingRight", this.i.a5);
        }
        /**
         * Gets or sets the top padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingTop() {
            return this.i.bg;
        }
        set separatorHorizontalPaddingTop(v) {
            this.i.bg = +v;
            this._a("separatorHorizontalPaddingTop", this.i.bg);
        }
        get actualSeparatorHorizontalPaddingTop() {
            return this.i.a6;
        }
        set actualSeparatorHorizontalPaddingTop(v) {
            this.i.a6 = +v;
            this._a("actualSeparatorHorizontalPaddingTop", this.i.a6);
        }
        /**
         * Gets or sets the bottom padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingBottom() {
            return this.i.bd;
        }
        set separatorHorizontalPaddingBottom(v) {
            this.i.bd = +v;
            this._a("separatorHorizontalPaddingBottom", this.i.bd);
        }
        get actualSeparatorHorizontalPaddingBottom() {
            return this.i.a3;
        }
        set actualSeparatorHorizontalPaddingBottom(v) {
            this.i.a3 = +v;
            this._a("actualSeparatorHorizontalPaddingBottom", this.i.a3);
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get orientation() {
            return this.i.o;
        }
        set orientation(v) {
            this.i.o = ensureEnum(ToolPanelOrientation_$type, v);
            this._a("orientation", enumToString(ToolPanelOrientation_$type, this.i.o));
        }
        /**
         * Gets or sets the row height for the panel.
        */
        get rowHeight() {
            return this.i.bn;
        }
        set rowHeight(v) {
            this.i.bn = +v;
            this._a("rowHeight", this.i.bn);
        }
        /**
         * Gets the actual row height used by the panel.
        */
        get actualRowHeight() {
            return this.i.a2;
        }
        /**
         * Gets or sets the mode used when displaying nested actions.
        */
        get nestedActionMode() {
            return this.i.e;
        }
        set nestedActionMode(v) {
            this.i.e = ensureEnum(NestedActionMode_$type, v);
            this._a("nestedActionMode", enumToString(NestedActionMode_$type, this.i.e));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolPanelComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        refresh() {
            this.i.dw();
        }
        getDesiredSize() {
            let iv = this.i.fk();
            return fromSize(iv);
        }
        /**
         * Forces any open sub menus to close.
        
        */
        closeSubmenus() {
            this.i.dc();
        }
        get contentRefreshed() {
            return this._contentRefreshed;
        }
        set contentRefreshed(ev) {
            if (this._contentRefreshed_wrapped !== null) {
                this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
                this._contentRefreshed_wrapped = null;
                this._contentRefreshed = null;
            }
            this._contentRefreshed = ev;
            this._contentRefreshed_wrapped = (o, e) => {
                let outerArgs = new IgcToolPanelContentRefreshedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeContentRefreshed) {
                    this.beforeContentRefreshed(this, outerArgs);
                }
                if (this._contentRefreshed) {
                    this._contentRefreshed(this, outerArgs);
                }
            };
            this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            ;
        }
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
    }
    IgcToolPanelComponent._observedAttributesIgcToolPanelComponent = null;
    IgcToolPanelComponent.htmlTagName = "igc-tool-panel";
    IgcToolPanelComponent._isElementRegistered = false;
    return IgcToolPanelComponent;
})();
