import { ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
import { WebComponentRenderer, PortalManager } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { CollectionAdapter } from "igniteui-webcomponents-core";
import { NotifyCollectionChangedAction } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { Toolbar } from "./Toolbar";
import { BaseControlTheme_$type } from "igniteui-webcomponents-core";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { ToolbarOrientation_$type } from './ToolbarOrientation';
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { ToolAction } from "./ToolAction";
import { IgcToolActionComponent } from "./igc-tool-action-component";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { IgcToolCommandEventArgs } from './igc-tool-command-event-args';
export let IgcToolbarComponent = /*@__PURE__*/ (() => {
    class IgcToolbarComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
            this.i.notifySizeChanged();
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
            this.i.notifySizeChanged();
        }
        get width() {
            return this._width;
        }
        /**
         * A collection of manually added actions for the editor.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let inner = coll._innerColl;
                inner.addListener((sender, e) => {
                    switch (e.action) {
                        case NotifyCollectionChangedAction.Add:
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            break;
                        case NotifyCollectionChangedAction.Replace:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            this._actionsAdapter.clearManualItems();
                            break;
                    }
                });
                this._actions = coll;
            }
            return this._actions;
        }
        constructor() {
            super();
            this._implementation = null;
            this.contentActions = [];
            /**
            * The actions actually present in the editor. Do not directly modify this array.
            * This array's contents can be modified by using the actions property or providing child content.
            */
            this.combinedActions = [];
            this._actions = null;
            this._actionsAdapter = null;
            this._disconnected = false;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this._autoGeneratedActions = null;
            this._actualActions = null;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this._portalManager = new PortalManager("toolbarContent");
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {}, this._portalManager);
            this._webComponentWrapper = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._webComponentWrapper);
            this._webComponentWrapper.setStyleProperty("display", "block");
            this._webComponentWrapper.setStyleProperty("width", "100%");
            this._webComponentWrapper.setStyleProperty("height", "100%");
            this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.combinedActions, (c) => c.i, (i) => {
                if (this._webComponentRenderer && this._webComponentRenderer.rootWrapper.getNativeElement().parentElement) {
                    i._styling(this, this, this);
                }
            }, (i) => { });
            this.i.provideContainer(this._webComponentRenderer);
            this.i.notifySizeChanged();
            let mut = new MutationObserver((list) => {
                for (var mutation of list) {
                    if (mutation.type == 'childList') {
                        this.updateContentProperties();
                    }
                }
            });
            mut.observe(this, {
                childList: true
            });
            this._webComponentRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        updateContentProperties() {
            this.contentActions.length = 0;
            for (var i = 0; i < this.children.length; i++) {
                if (this.children[i] instanceof IgcToolActionComponent) {
                    this.contentActions.push(this.children[i]);
                }
            }
            if (this._actionsAdapter != null) {
                this._actionsAdapter.notifyContentChanged();
            }
        }
        createImplementation() {
            return new Toolbar();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-toolbar");
            this.classList.add("igc-toolbar");
            var rootWrapper = this._webComponentRenderer.rootWrapper;
            var rootElement = rootWrapper.getNativeElement();
            this.appendChild(rootElement);
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            for (let i = 0; i < this.combinedActions.length; i++) {
                let s = this.combinedActions[i];
                s._styling(this, this, this);
            }
            this.afterContentInit();
        }
        afterContentInit() {
            this.i.notifySizeChanged();
        }
        /**
         * Gets or sets a target component which the toolbar can request actions from.
         */
        get target() {
            var ret = this.i.target;
            if (!ret) {
                return ret;
            }
            if (ret.externalObject != null) {
                ret = ret.externalObject;
            }
            return ret;
        }
        set target(v) {
            if (v && v._implementation) {
                v = v._implementation;
            }
            this.i.target = v;
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcToolbarComponent._observedAttributesIgcToolbarComponent == null) {
                let names = getAllPropertyNames(IgcToolbarComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolbarComponent._observedAttributesIgcToolbarComponent = names;
            }
            return IgcToolbarComponent._observedAttributesIgcToolbarComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcToolbarComponent._isElementRegistered) {
                IgcToolbarComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcToolbarComponent.htmlTagName, IgcToolbarComponent);
            }
        }
        /**
     * Gets or sets the base built in theme to use for the checkbox.
    */
        get baseTheme() {
            return this.i.p;
        }
        set baseTheme(v) {
            this.i.p = ensureEnum(BaseControlTheme_$type, v);
            this._a("baseTheme", enumToString(BaseControlTheme_$type, this.i.p));
        }
        /**
         * Gets or sets the data or data source instance to which to bind the grid.
        */
        get autoGeneratedActions() {
            if (this._autoGeneratedActions === null) {
                let coll = new IgcToolActionCollection();
                let innerColl = this.i.autoGeneratedActions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._autoGeneratedActions = coll._fromInner(innerColl);
                this.i.autoGeneratedActions = innerColl;
            }
            return this._autoGeneratedActions;
        }
        set autoGeneratedActions(v) {
            if (this._autoGeneratedActions !== null) {
                this._autoGeneratedActions._setSyncTarget(null);
                this._autoGeneratedActions = null;
            }
            let coll = new IgcToolActionCollection();
            this._autoGeneratedActions = coll._fromOuter(v);
            let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            let innerColl = this.i.autoGeneratedActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._autoGeneratedActions._setSyncTarget(syncColl);
            this.i.autoGeneratedActions = innerColl;
        }
        /**
         * Gets or sets the data or data source instance to which to bind the grid.
        */
        get actualActions() {
            if (this._actualActions === null) {
                let coll = new IgcToolActionCollection();
                let innerColl = this.i.actualActions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actualActions = coll._fromInner(innerColl);
                this.i.actualActions = innerColl;
            }
            return this._actualActions;
        }
        set actualActions(v) {
            if (this._actualActions !== null) {
                this._actualActions._setSyncTarget(null);
                this._actualActions = null;
            }
            let coll = new IgcToolActionCollection();
            this._actualActions = coll._fromOuter(v);
            let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            let innerColl = this.i.actualActions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actualActions._setSyncTarget(syncColl);
            this.i.actualActions = innerColl;
        }
        /**
         * Gets or sets the orientation of the toolbar.
        */
        get orientation() {
            return this.i.k;
        }
        set orientation(v) {
            this.i.k = ensureEnum(ToolbarOrientation_$type, v);
            this._a("orientation", enumToString(ToolbarOrientation_$type, this.i.k));
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get backgroundColor() {
            return brushToString(this.i.cw);
        }
        set backgroundColor(v) {
            this.i.cw = stringToBrush(v);
            this._a("backgroundColor", brushToString(this.i.cw));
        }
        /**
         * Gets or sets the background color for submenus.
        */
        get submenuBackgroundColor() {
            return brushToString(this.i.c4);
        }
        set submenuBackgroundColor(v) {
            this.i.c4 = stringToBrush(v);
            this._a("submenuBackgroundColor", brushToString(this.i.c4));
        }
        /**
         * Gets or sets the group header background color.
        */
        get groupHeaderBackgroundColor() {
            return brushToString(this.i.cy);
        }
        set groupHeaderBackgroundColor(v) {
            this.i.cy = stringToBrush(v);
            this._a("groupHeaderBackgroundColor", brushToString(this.i.cy));
        }
        /**
         * Gets or sets the separator background color.
        */
        get separatorBackgroundColor() {
            return brushToString(this.i.c3);
        }
        set separatorBackgroundColor(v) {
            this.i.c3 = stringToBrush(v);
            this._a("separatorBackgroundColor", brushToString(this.i.c3));
        }
        /**
         * Gets or sets the hover background color.
        */
        get hoverBackgroundColor() {
            return brushToString(this.i.c0);
        }
        set hoverBackgroundColor(v) {
            this.i.c0 = stringToBrush(v);
            this._a("hoverBackgroundColor", brushToString(this.i.c0));
        }
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get textColor() {
            return brushToString(this.i.c5);
        }
        set textColor(v) {
            this.i.c5 = stringToBrush(v);
            this._a("textColor", brushToString(this.i.c5));
        }
        /**
         * Gets or sets the color to use for the text in disabled tool actions.
        */
        get disabledTextColor() {
            return brushToString(this.i.cx);
        }
        set disabledTextColor(v) {
            this.i.cx = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.cx));
        }
        /**
         * Gets or sets the text color for group header tool actions.
        */
        get groupHeaderTextColor() {
            return brushToString(this.i.cz);
        }
        set groupHeaderTextColor(v) {
            this.i.cz = stringToBrush(v);
            this._a("groupHeaderTextColor", brushToString(this.i.cz));
        }
        /**
         * Gets or sets the tooltip text color.
        */
        get toolTipTextColor() {
            return brushToString(this.i.c7);
        }
        set toolTipTextColor(v) {
            this.i.c7 = stringToBrush(v);
            this._a("toolTipTextColor", brushToString(this.i.c7));
        }
        /**
         * Gets or sets the tooltip background color.
        */
        get toolTipBackgroundColor() {
            return brushToString(this.i.c6);
        }
        set toolTipBackgroundColor(v) {
            this.i.c6 = stringToBrush(v);
            this._a("toolTipBackgroundColor", brushToString(this.i.c6));
        }
        /**
         * Gets or sets the tooltip corner radius.
        */
        get toolTipCornerRadius() {
            return this.i.av;
        }
        set toolTipCornerRadius(v) {
            this.i.av = +v;
            this._a("toolTipCornerRadius", this.i.av);
        }
        /**
         * Gets or sets the shadow elevation for the tooltip.
        */
        get toolTipElevation() {
            return this.i.aw;
        }
        set toolTipElevation(v) {
            this.i.aw = +v;
            this._a("toolTipElevation", this.i.aw);
        }
        /**
         * Gets or sets the fill color of icons in the ToolPanel.
        */
        get iconFill() {
            return brushToString(this.i.c1);
        }
        set iconFill(v) {
            this.i.c1 = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.c1));
        }
        /**
         * Gets or sets the stroke color of icons in the ToolPanel.
        */
        get iconStroke() {
            return brushToString(this.i.c2);
        }
        set iconStroke(v) {
            this.i.c2 = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.c2));
        }
        /**
         * Gets or Sets the property name that contains the values.
        */
        get cellTextStyle() {
            if (this.i.s == null) {
                return null;
            }
            return this.i.s.fontString;
        }
        set cellTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.s = fi;
            this._a("cellTextStyle", this.i.s != null ? this.i.s.fontString : "");
        }
        /**
         * Gets or sets the left padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingLeft() {
            return this.i.as;
        }
        set separatorVerticalPaddingLeft(v) {
            this.i.as = +v;
            this._a("separatorVerticalPaddingLeft", this.i.as);
        }
        /**
         * Gets or sets the right padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingRight() {
            return this.i.at;
        }
        set separatorVerticalPaddingRight(v) {
            this.i.at = +v;
            this._a("separatorVerticalPaddingRight", this.i.at);
        }
        /**
         * Gets or sets the top padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingTop() {
            return this.i.au;
        }
        set separatorVerticalPaddingTop(v) {
            this.i.au = +v;
            this._a("separatorVerticalPaddingTop", this.i.au);
        }
        /**
         * Gets or sets the bottom padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingBottom() {
            return this.i.ar;
        }
        set separatorVerticalPaddingBottom(v) {
            this.i.ar = +v;
            this._a("separatorVerticalPaddingBottom", this.i.ar);
        }
        /**
         * Gets or sets the left padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingLeft() {
            return this.i.ao;
        }
        set separatorHorizontalPaddingLeft(v) {
            this.i.ao = +v;
            this._a("separatorHorizontalPaddingLeft", this.i.ao);
        }
        /**
         * Gets or sets the right padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingRight() {
            return this.i.ap;
        }
        set separatorHorizontalPaddingRight(v) {
            this.i.ap = +v;
            this._a("separatorHorizontalPaddingRight", this.i.ap);
        }
        /**
         * Gets or sets the top padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingTop() {
            return this.i.aq;
        }
        set separatorHorizontalPaddingTop(v) {
            this.i.aq = +v;
            this._a("separatorHorizontalPaddingTop", this.i.aq);
        }
        /**
         * Gets or sets the bottom padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingBottom() {
            return this.i.an;
        }
        set separatorHorizontalPaddingBottom(v) {
            this.i.an = +v;
            this._a("separatorHorizontalPaddingBottom", this.i.an);
        }
        /**
         * Gets or sets if the tool actions should display their submenus on mouse hover.
        */
        get showOnHover() {
            return this.i.ac;
        }
        set showOnHover(v) {
            this.i.ac = ensureBool(v);
            this._a("showOnHover", this.i.ac);
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.autoGeneratedActions != null && this.autoGeneratedActions.findByName && this.autoGeneratedActions.findByName(name)) {
                return this.autoGeneratedActions.findByName(name);
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            if (this.actualActions != null && this.actualActions.findByName && this.actualActions.findByName(name)) {
                return this.actualActions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolbarComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        /**
     * Forces any open sub menus to close.
    
    */
        closeSubmenus() {
            this.i.b2();
        }
        onCommandStateChanged(id, type, value) {
            this.i.onCommandStateChanged(id, (type == null ? null : type), value);
        }
        registerIconFromText(iconCollection, iconName, iconText) {
            this.i.ch(iconCollection, iconName, iconText);
        }
        getIconFromCache(iconCollection, iconName) {
            let iv = this.i.a9(iconCollection, iconName);
            return (iv);
        }
        /**
         * Event raised when a command is triggered in the toolbar.
        */
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
    }
    IgcToolbarComponent._observedAttributesIgcToolbarComponent = null;
    IgcToolbarComponent.htmlTagName = "igc-toolbar";
    IgcToolbarComponent._isElementRegistered = false;
    return IgcToolbarComponent;
})();
