import { brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { WebComponentRenderer, PortalManager } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { XExpansionPanel } from "./XExpansionPanel";
import { IgcOnExpandedEventArgs } from './igc-on-expanded-event-args';
import { IgcOnCollapsedEventArgs } from './igc-on-collapsed-event-args';
import { IgcHTMLElement } from "igniteui-webcomponents-core";
export let IgcXExpansionPanelComponent = /*@__PURE__*/ (() => {
    class IgcXExpansionPanelComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
        }
        get width() {
            return this._width;
        }
        constructor() {
            super();
            this._implementation = null;
            this._disconnected = false;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onCollapsed = null;
            this._onCollapsed_wrapped = null;
            this._onExpanded = null;
            this._onExpanded_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._portalManager = new PortalManager("iconContent");
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {}, this._portalManager);
            this._mainDiv = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._mainDiv);
            let header = this._webComponentRenderer.createElement("div");
            header.setAttribute("key", "expansionPanelHeaderArea");
            let contentOuter = this._webComponentRenderer.createElement("div");
            contentOuter.setAttribute("key", "expansionPanelContentAreaOuter");
            let contentInner = this._webComponentRenderer.createElement("div");
            contentInner.setAttribute("key", "expansionPanelContentAreaInner");
            contentOuter.append(contentInner);
            this._mainDiv.append(header);
            this._mainDiv.append(contentOuter);
            let mut = new MutationObserver((list) => {
                for (var mutation of list) {
                    if (mutation.type == 'childList') {
                        for (var i = 0; i < this.childNodes.length; i++) {
                            var child = this.childNodes[i];
                            if (child != this._mainDiv.getNativeElement() &&
                                child !== header.getNativeElement() &&
                                child !== contentOuter.getNativeElement() &&
                                child !== contentInner.getNativeElement()) {
                                //TODO: maybe hide and clone this, allow for dynamic removal of content, etc.
                                child.remove();
                                contentInner.append(this._webComponentRenderer.getWrapper(child));
                            }
                        }
                    }
                }
            });
            mut.observe(this, {
                childList: true
            });
            this.i.provideRenderer(this._webComponentRenderer);
            this.i.provideHeader(this._webComponentRenderer.getWrapper(header));
            this.i.provideContentAreaOuter(this._webComponentRenderer.getWrapper(contentOuter));
            this.i.provideContentAreaInner(this._webComponentRenderer.getWrapper(contentInner));
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        createImplementation() {
            return new XExpansionPanel();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-expansion-panel");
            this.classList.add("igr-expansion-panel");
            this.appendChild(this._mainDiv.getNativeElement());
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            this.afterContentInit();
        }
        afterContentInit() {
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcXExpansionPanelComponent._observedAttributesIgcXExpansionPanelComponent == null) {
                let names = getAllPropertyNames(IgcXExpansionPanelComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcXExpansionPanelComponent._observedAttributesIgcXExpansionPanelComponent = names;
            }
            return IgcXExpansionPanelComponent._observedAttributesIgcXExpansionPanelComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcXExpansionPanelComponent._isElementRegistered) {
                IgcXExpansionPanelComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcXExpansionPanelComponent.htmlTagName, IgcXExpansionPanelComponent);
            }
        }
        /**
     * Gets / sets the caption to display in the header area.
    */
        get caption() {
            return this.i.ab;
        }
        set caption(v) {
            this.i.ab = v;
        }
        /**
         * Gets or sets the color to use for the header caption text.
        */
        get captionTextColor() {
            return brushToString(this.i.bq);
        }
        set captionTextColor(v) {
            this.i.bq = stringToBrush(v);
            this._a("captionTextColor", brushToString(this.i.bq));
        }
        /**
         * Gets or sets the color to use for the actual header caption text.
        */
        get actualCaptionTextColor() {
            return brushToString(this.i.bj);
        }
        set actualCaptionTextColor(v) {
            this.i.bj = stringToBrush(v);
            this._a("actualCaptionTextColor", brushToString(this.i.bj));
        }
        /**
         * Gets or sets the color to use for the header caption text when the panel is collapsed.
        */
        get captionCollapsedTextColor() {
            return brushToString(this.i.bo);
        }
        set captionCollapsedTextColor(v) {
            this.i.bo = stringToBrush(v);
            this._a("captionCollapsedTextColor", brushToString(this.i.bo));
        }
        /**
         * Gets or sets the color to use for the header caption text when the panel is expanded.
        */
        get captionExpandedTextColor() {
            return brushToString(this.i.bp);
        }
        set captionExpandedTextColor(v) {
            this.i.bp = stringToBrush(v);
            this._a("captionExpandedTextColor", brushToString(this.i.bp));
        }
        /**
         * Gets / sets the description to display in the header area.
        */
        get description() {
            return this.i.ah;
        }
        set description(v) {
            this.i.ah = v;
        }
        /**
         * Gets or sets the color to use for the header description text.
        */
        get descriptionTextColor() {
            return brushToString(this.i.bz);
        }
        set descriptionTextColor(v) {
            this.i.bz = stringToBrush(v);
            this._a("descriptionTextColor", brushToString(this.i.bz));
        }
        /**
         * Gets or sets the color to use for the actual header description text.
        */
        get actualDescriptionTextColor() {
            return brushToString(this.i.bk);
        }
        set actualDescriptionTextColor(v) {
            this.i.bk = stringToBrush(v);
            this._a("actualDescriptionTextColor", brushToString(this.i.bk));
        }
        /**
         * Gets or sets the color to use for the header description text when the panel is collapsed.
        */
        get descriptionCollapsedTextColor() {
            return brushToString(this.i.bx);
        }
        set descriptionCollapsedTextColor(v) {
            this.i.bx = stringToBrush(v);
            this._a("descriptionCollapsedTextColor", brushToString(this.i.bx));
        }
        /**
         * Gets or sets the color to use for the header description text when the panel is expanded.
        */
        get descriptionExpandedTextColor() {
            return brushToString(this.i.by);
        }
        set descriptionExpandedTextColor(v) {
            this.i.by = stringToBrush(v);
            this._a("descriptionExpandedTextColor", brushToString(this.i.by));
        }
        /**
         * Gets or sets the elevation to use for the button regardless of type.
        */
        get elevation() {
            return this.i.s;
        }
        set elevation(v) {
            this.i.s = +v;
            this._a("elevation", this.i.s);
        }
        /**
         * Gets or sets the elevation of the border shadow
        */
        get actualElevation() {
            return this.i.q;
        }
        set actualElevation(v) {
            this.i.q = +v;
            this._a("actualElevation", this.i.q);
        }
        /**
         * Gets / sets the expanded state of the panel.
        */
        get expanded() {
            return this.i.n;
        }
        set expanded(v) {
            this.i.n = ensureBool(v);
            this._a("expanded", this.i.n);
        }
        /**
         * Gets or sets the color to use for the background of the header.
        */
        get headerBackgroundColor() {
            return brushToString(this.i.b6);
        }
        set headerBackgroundColor(v) {
            this.i.b6 = stringToBrush(v);
            this._a("headerBackgroundColor", brushToString(this.i.b6));
        }
        /**
         * Gets or sets the color to use for the actual background of the header.
        */
        get actualHeaderBackgroundColor() {
            return brushToString(this.i.bl);
        }
        set actualHeaderBackgroundColor(v) {
            this.i.bl = stringToBrush(v);
            this._a("actualHeaderBackgroundColor", brushToString(this.i.bl));
        }
        /**
         * Gets or sets the color to use for the background of the header when the panel is expanded.
        */
        get headerCollapsedBackgroundColor() {
            return brushToString(this.i.b7);
        }
        set headerCollapsedBackgroundColor(v) {
            this.i.b7 = stringToBrush(v);
            this._a("headerCollapsedBackgroundColor", brushToString(this.i.b7));
        }
        /**
         * Gets or sets the color to use for the background of the header when the panel is expanded.
        */
        get headerExpandedBackgroundColor() {
            return brushToString(this.i.b8);
        }
        set headerExpandedBackgroundColor(v) {
            this.i.b8 = stringToBrush(v);
            this._a("headerExpandedBackgroundColor", brushToString(this.i.b8));
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualUmbraShadowColor() {
            return brushToString(this.i.bn);
        }
        set actualUmbraShadowColor(v) {
            this.i.bn = stringToBrush(v);
            this._a("actualUmbraShadowColor", brushToString(this.i.bn));
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualPenumbraShadowColor() {
            return brushToString(this.i.bm);
        }
        set actualPenumbraShadowColor(v) {
            this.i.bm = stringToBrush(v);
            this._a("actualPenumbraShadowColor", brushToString(this.i.bm));
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualAmbientShadowColor() {
            return brushToString(this.i.bi);
        }
        set actualAmbientShadowColor(v) {
            this.i.bi = stringToBrush(v);
            this._a("actualAmbientShadowColor", brushToString(this.i.bi));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("XExpansionPanelComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        /**
     * Collapses the contents of the expansion panel.
    
    */
        collapse() {
            this.i.ar();
        }
        /**
         * Expands the contents of the expansion panel.
        
        */
        expand() {
            this.i.au();
        }
        /**
         * Toggles the expanded state of the expansion panel.
        
        */
        toggle() {
            this.i.a2();
        }
        /**
         * Fires when the expansion panel is collapsed.
        */
        get onCollapsed() {
            return this._onCollapsed;
        }
        set onCollapsed(ev) {
            if (this._onCollapsed_wrapped !== null) {
                this.i.onCollapsed = delegateRemove(this.i.onCollapsed, this._onCollapsed_wrapped);
                this._onCollapsed_wrapped = null;
                this._onCollapsed = null;
            }
            this._onCollapsed = ev;
            this._onCollapsed_wrapped = (o, e) => {
                let outerArgs = new IgcOnCollapsedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCollapsed) {
                    this.beforeOnCollapsed(this, outerArgs);
                }
                if (this._onCollapsed) {
                    this._onCollapsed(this, outerArgs);
                }
            };
            this.i.onCollapsed = delegateCombine(this.i.onCollapsed, this._onCollapsed_wrapped);
            ;
        }
        get onExpanded() {
            return this._onExpanded;
        }
        set onExpanded(ev) {
            if (this._onExpanded_wrapped !== null) {
                this.i.onExpanded = delegateRemove(this.i.onExpanded, this._onExpanded_wrapped);
                this._onExpanded_wrapped = null;
                this._onExpanded = null;
            }
            this._onExpanded = ev;
            this._onExpanded_wrapped = (o, e) => {
                let outerArgs = new IgcOnExpandedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnExpanded) {
                    this.beforeOnExpanded(this, outerArgs);
                }
                if (this._onExpanded) {
                    this._onExpanded(this, outerArgs);
                }
            };
            this.i.onExpanded = delegateCombine(this.i.onExpanded, this._onExpanded_wrapped);
            ;
        }
    }
    IgcXExpansionPanelComponent._observedAttributesIgcXExpansionPanelComponent = null;
    IgcXExpansionPanelComponent.htmlTagName = "igc-x-expansion-panel";
    IgcXExpansionPanelComponent._isElementRegistered = false;
    return IgcXExpansionPanelComponent;
})();
