import { __extends } from "tslib";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { Visibility_$type } from "igniteui-webcomponents-core";
import { IgcToolCommandEventArgs } from "./igc-tool-command-event-args";
import { IgcToolActionPerformedEventArgs } from "./igc-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-webcomponents-core";
import { NamePatcher, getAllPropertyNames, fromSpinal, toSpinal, brushToString, stringToBrush, ensureBool, ensureEnum, enumToString, initializePropertiesFromCss } from "igniteui-webcomponents-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
/**
 * Represents base class for all tool actions in toolbar
*/
var IgcToolActionComponent = /** @class */ /*@__PURE__*/ (function (_super) {
    __extends(IgcToolActionComponent, _super);
    function IgcToolActionComponent() {
        var _this = _super.call(this) || this;
        _this._settingAttributes = false;
        _this._attached = false;
        _this._queuedSetAttributes = [];
        _this._updatingFromAttribute = false;
        _this._actions = null;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onCommand = null;
        _this._onCommand_wrapped = null;
        _this._performed = null;
        _this._performed_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._implementation.externalObject = _this;
        _this.onImplementationCreated();
        return _this;
    }
    IgcToolActionComponent.prototype.createImplementation = function () {
        return null;
    };
    Object.defineProperty(IgcToolActionComponent.prototype, "i", {
        /**
         * @hidden
         */
        get: function () {
            return this._implementation;
        } /**
                                     * @hidden
                                     */,
        enumerable: false,
        configurable: true
    });
    IgcToolActionComponent._createFromInternal = function (internal) {
        if (!internal) {
            return null;
        }
        if (!internal.$type) {
            return null;
        }
        var name = internal.$type.name;
        var externalName = "Igc" + name + "Component";
        if (!TypeRegistrar.isRegistered(externalName)) {
            return null;
        }
        return TypeRegistrar.create(externalName);
    };
    IgcToolActionComponent.prototype.onImplementationCreated = function () {
    };
    IgcToolActionComponent.prototype._enqueueSetAttribute = function (attrName, attrValue) {
        this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
    };
    IgcToolActionComponent.prototype._flushQueuedAttributes = function () {
        this._settingAttributes = true;
        for (var i = 0; i < this._queuedSetAttributes.length; i++) {
            this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
        }
        this._settingAttributes = false;
        this._queuedSetAttributes.length = 0;
    };
    IgcToolActionComponent.prototype._a = function (attrName, attrValue) {
        if (this._updatingFromAttribute) {
            return;
        }
        if (attrValue) {
            attrValue = attrValue.toString();
        }
        this._settingAttributes = true;
        attrName = toSpinal(attrName);
        if (this._attached) {
            this.setAttribute(attrName, attrValue);
        }
        else {
            this._enqueueSetAttribute(attrName, attrValue);
        }
        this._settingAttributes = false;
    };
    IgcToolActionComponent.prototype.connectedCallback = function () {
        if (_super.prototype["connectedCallback"]) {
            _super.prototype["connectedCallback"].call(this);
        }
        if (this.i.connectedCallback) {
            this.i.connectedCallback();
        }
        if (!this._attached) {
            this._attached = true;
            this._flushQueuedAttributes();
        }
    };
    IgcToolActionComponent.prototype.disconnectedCallback = function () {
        if (_super.prototype["disconnectedCallback"]) {
            _super.prototype["disconnectedCallback"].call(this);
        }
        if (this.i.disconnectedCallback) {
            this.i.disconnectedCallback();
        }
        if (this._attached) {
            this._attached = false;
        }
    };
    Object.defineProperty(IgcToolActionComponent, "observedAttributes", {
        get: function () {
            if (IgcToolActionComponent._observedAttributesIgcToolActionComponent == null) {
                var names = getAllPropertyNames(IgcToolActionComponent);
                for (var i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolActionComponent._observedAttributesIgcToolActionComponent = names;
            }
            return IgcToolActionComponent._observedAttributesIgcToolActionComponent;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolActionComponent.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
        if (this._settingAttributes) {
            return;
        }
        var setName = fromSpinal(name);
        this._updatingFromAttribute = true;
        this[setName] = newValue;
        this._updatingFromAttribute = false;
    };
    Object.defineProperty(IgcToolActionComponent.prototype, "id", {
        get: function () {
            return this.i.cs;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actions", {
        /**
         * Gets the sub actions associated with this tool action.
        */
        get: function () {
            if (this._actions === null) {
                var coll = new IgcToolActionCollection();
                var innerColl = this.i.actions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actions = coll._fromInner(innerColl);
            }
            return this._actions;
        },
        set: function (v) {
            if (this._actions !== null) {
                this._actions._setSyncTarget(null);
                this._actions = null;
            }
            var coll = new IgcToolActionCollection();
            this._actions = coll._fromOuter(v);
            var syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            var innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actions._setSyncTarget(syncColl);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "width", {
        /**
         * Gets or sets the width to use for this tool action.
        */
        get: function () {
            return this.i.bi;
        },
        set: function (v) {
            this.i.bi = +v;
            this._a("width", this.i.bi);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "parentId", {
        get: function () {
            return this.i.c1;
        },
        set: function (v) {
            this.i.c1 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "beforeId", {
        /**
         * Gets or sets the Id of the action that this action will be inserted before.
         * Subsequent actions will continue to insert before the target action, following
         * the original action that set this property. If using this property do not use the
         * AfterId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get: function () {
            return this.i.b9;
        },
        set: function (v) {
            this.i.b9 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "afterId", {
        /**
         * Gets or sets the Id of the action that this action will be inserted after.
         * Subsequent actions will continue to insert after the target action, following
         * the original action that set this property. If using this property do not use the
         * BeforeId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get: function () {
            return this.i.b6;
        },
        set: function (v) {
            this.i.b6 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "overlayId", {
        /**
         * Sets to the Id of the action that this action will overlay itself on to.
         * The purpose of this property is to allow users to override actions automatically generated by
         * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
         * with them that can be used for the OverlayId.
         * These overlaid actions will
         * not
         * replace the generated one but will have its properties
         * overlaid onto the generated actions.
        */
        get: function () {
            return this.i.cw;
        },
        set: function (v) {
            this.i.cw = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "commandId", {
        get: function () {
            return this.i.cd;
        },
        set: function (v) {
            this.i.cd = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "commandArgument", {
        get: function () {
            return this.i.ca;
        },
        set: function (v) {
            this.i.ca = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "commandArgumentValue", {
        get: function () {
            return this.i.bk;
        },
        set: function (v) {
            this.i.bk = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "paddingLeft", {
        get: function () {
            return this.i.be;
        },
        set: function (v) {
            this.i.be = +v;
            this._a("paddingLeft", this.i.be);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualPaddingLeft", {
        get: function () {
            return this.i.a2;
        },
        set: function (v) {
            this.i.a2 = +v;
            this._a("actualPaddingLeft", this.i.a2);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "paddingRight", {
        get: function () {
            return this.i.bf;
        },
        set: function (v) {
            this.i.bf = +v;
            this._a("paddingRight", this.i.bf);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualPaddingRight", {
        get: function () {
            return this.i.a3;
        },
        set: function (v) {
            this.i.a3 = +v;
            this._a("actualPaddingRight", this.i.a3);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "paddingBottom", {
        get: function () {
            return this.i.bd;
        },
        set: function (v) {
            this.i.bd = +v;
            this._a("paddingBottom", this.i.bd);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualPaddingBottom", {
        get: function () {
            return this.i.a1;
        },
        set: function (v) {
            this.i.a1 = +v;
            this._a("actualPaddingBottom", this.i.a1);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "paddingTop", {
        get: function () {
            return this.i.bg;
        },
        set: function (v) {
            this.i.bg = +v;
            this._a("paddingTop", this.i.bg);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualPaddingTop", {
        get: function () {
            return this.i.a4;
        },
        set: function (v) {
            this.i.a4 = +v;
            this._a("actualPaddingTop", this.i.a4);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualHoverBackground", {
        /**
         * Gets the actual hover background used by the tool action.
        */
        get: function () {
            return brushToString(this.i.eq);
        },
        set: function (v) {
            this.i.eq = stringToBrush(v);
            this._a("actualHoverBackground", brushToString(this.i.eq));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "hoverBackground", {
        /**
         * Gets or sets the hover background for this tool action.
        */
        get: function () {
            return brushToString(this.i.ew);
        },
        set: function (v) {
            this.i.ew = stringToBrush(v);
            this._a("hoverBackground", brushToString(this.i.ew));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualBackground", {
        /**
         * Gets the actual background used by the tool action.
        */
        get: function () {
            return brushToString(this.i.eo);
        },
        set: function (v) {
            this.i.eo = stringToBrush(v);
            this._a("actualBackground", brushToString(this.i.eo));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "background", {
        /**
         * Gets or sets the background color of the tool action.
        */
        get: function () {
            return brushToString(this.i.eu);
        },
        set: function (v) {
            this.i.eu = stringToBrush(v);
            this._a("background", brushToString(this.i.eu));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualTextColor", {
        /**
         * Gets the actual label text color.
        */
        get: function () {
            return brushToString(this.i.et);
        },
        set: function (v) {
            this.i.et = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.et));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "textColor", {
        /**
         * Gets or sets the label text color.
        */
        get: function () {
            return brushToString(this.i.ez);
        },
        set: function (v) {
            this.i.ez = stringToBrush(v);
            this._a("textColor", brushToString(this.i.ez));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualDisabledTextColor", {
        /**
         * Gets the actual label text color when the tool is disabled.
        */
        get: function () {
            return brushToString(this.i.ep);
        },
        set: function (v) {
            this.i.ep = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.ep));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "disabledTextColor", {
        /**
         * Gets or sets the label text color when the tool is disabled.
        */
        get: function () {
            return brushToString(this.i.ev);
        },
        set: function (v) {
            this.i.ev = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.ev));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualTextStyle", {
        /**
         * Gets the actual font used.
        */
        get: function () {
            if (this.i.r == null) {
                return null;
            }
            return this.i.r.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.r = fi;
            this._a("actualTextStyle", this.i.r != null ? this.i.r.fontString : "");
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "textStyle", {
        /**
         * Gets or sets the font to use.
        */
        get: function () {
            if (this.i.t == null) {
                return null;
            }
            return this.i.t.fontString;
        },
        set: function (v) {
            var fi = new FontInfo();
            fi.fontString = v;
            this.i.t = fi;
            this._a("textStyle", this.i.t != null ? this.i.t.fontString : "");
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "isDisabled", {
        get: function () {
            return this.i.ah;
        },
        set: function (v) {
            this.i.ah = ensureBool(v);
            this._a("isDisabled", this.i.ah);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "name", {
        get: function () {
            return this.i.cu;
        },
        set: function (v) {
            this.i.cu = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "title", {
        /**
         * Gets or sets the title of the tool.
        */
        get: function () {
            return this.i.da;
        },
        set: function (v) {
            this.i.da = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "subtitle", {
        /**
         * Gets or sets the subtitle of the tool.
        */
        get: function () {
            return this.i.c6;
        },
        set: function (v) {
            this.i.c6 = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconName", {
        get: function () {
            return this.i.cl;
        },
        set: function (v) {
            this.i.cl = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconCollectionName", {
        get: function () {
            return this.i.ci;
        },
        set: function (v) {
            this.i.ci = v;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconViewBoxLeft", {
        /**
         * Gets or sets the icon viewbox left.
        */
        get: function () {
            return this.i.ba;
        },
        set: function (v) {
            this.i.ba = +v;
            this._a("iconViewBoxLeft", this.i.ba);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconViewBoxTop", {
        /**
         * Gets or sets the icon viewbox top.
        */
        get: function () {
            return this.i.bb;
        },
        set: function (v) {
            this.i.bb = +v;
            this._a("iconViewBoxTop", this.i.bb);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconViewBoxWidth", {
        /**
         * Gets or sets the icon viewbox width.
        */
        get: function () {
            return this.i.bc;
        },
        set: function (v) {
            this.i.bc = +v;
            this._a("iconViewBoxWidth", this.i.bc);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconViewBoxHeight", {
        /**
         * Gets or sets the icon viewbox height.
        */
        get: function () {
            return this.i.a9;
        },
        set: function (v) {
            this.i.a9 = +v;
            this._a("iconViewBoxHeight", this.i.a9);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconFill", {
        /**
         * Gets or sets the icon fill color.
        */
        get: function () {
            return brushToString(this.i.ex);
        },
        set: function (v) {
            this.i.ex = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.ex));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualIconFill", {
        /**
         * Gets the actual icon fill color.
        */
        get: function () {
            return brushToString(this.i.er);
        },
        set: function (v) {
            this.i.er = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.er));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "iconStroke", {
        /**
         * Gets or sets the icon stroke color.
        */
        get: function () {
            return brushToString(this.i.ey);
        },
        set: function (v) {
            this.i.ey = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.ey));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "actualIconStroke", {
        /**
         * Gets the actual icon stroke color.
        */
        get: function () {
            return brushToString(this.i.es);
        },
        set: function (v) {
            this.i.es = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.es));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "visibility", {
        /**
         * Gets or sets the tool action visibility.
        */
        get: function () {
            return this.i.e1;
        },
        set: function (v) {
            this.i.e1 = ensureEnum(Visibility_$type, v);
            this._a("visibility", enumToString(Visibility_$type, this.i.e1));
        },
        enumerable: false,
        configurable: true
    });
    IgcToolActionComponent.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
            return this.actions.findByName(name);
        }
        return null;
    };
    Object.defineProperty(IgcToolActionComponent.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolActionComponent.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgcToolActionComponent.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("ToolActionComponent");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    IgcToolActionComponent.prototype.cleanUp = function () {
        this.i.dd();
    };
    IgcToolActionComponent.prototype.fromInfo = function (info) {
        this.i.dp(info);
    };
    IgcToolActionComponent.prototype.openSubMenu = function () {
        this.i.d2();
    };
    IgcToolActionComponent.prototype.closeSubmenu = function () {
        this.i.df();
    };
    Object.defineProperty(IgcToolActionComponent.prototype, "onCommand", {
        get: function () {
            return this._onCommand;
        },
        set: function (ev) {
            var _this = this;
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = function (o, e) {
                var outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnCommand) {
                    _this.beforeOnCommand(_this, outerArgs);
                }
                if (_this._onCommand) {
                    _this._onCommand(_this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcToolActionComponent.prototype, "performed", {
        get: function () {
            return this._performed;
        },
        set: function (ev) {
            var _this = this;
            if (this._performed_wrapped !== null) {
                this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
                this._performed_wrapped = null;
                this._performed = null;
            }
            this._performed = ev;
            this._performed_wrapped = function (o, e) {
                var outerArgs = new IgcToolActionPerformedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforePerformed) {
                    _this.beforePerformed(_this, outerArgs);
                }
                if (_this._performed) {
                    _this._performed(_this, outerArgs);
                }
            };
            this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    IgcToolActionComponent._observedAttributesIgcToolActionComponent = null;
    return IgcToolActionComponent;
}(IgcHTMLElement));
export { IgcToolActionComponent };
