import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { Visibility } from "igniteui-webcomponents-core";
import { IgcToolCommandEventArgs } from "./igc-tool-command-event-args";
import { IgcToolActionPerformedEventArgs } from "./igc-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { ToolActionInfo } from "igniteui-webcomponents-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export declare abstract class IgcToolActionComponent extends IgcHTMLElement {
    protected createImplementation(): ToolAction;
    protected _implementation: any;
    /**
     * @hidden
     */
    get i(): ToolAction; /**
                                 * @hidden
                                 */
    static _createFromInternal(internal: any): IgcToolActionComponent;
    private onImplementationCreated;
    constructor();
    private _settingAttributes;
    protected _attached: boolean;
    private _queuedSetAttributes;
    protected _enqueueSetAttribute(attrName: string, attrValue: string): void;
    protected _flushQueuedAttributes(): void;
    protected _a(attrName: string, attrValue: any): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    private static _observedAttributesIgcToolActionComponent;
    static get observedAttributes(): string[];
    private _updatingFromAttribute;
    attributeChangedCallback(name: string, oldValue: string, newValue: string): void;
    get id(): string;
    private _actions;
    /**
     * Gets the sub actions associated with this tool action.
    */
    get actions(): IgcToolActionCollection;
    set actions(v: IgcToolActionCollection);
    /**
     * Gets or sets the width to use for this tool action.
    */
    get width(): number;
    set width(v: number);
    get parentId(): string;
    set parentId(v: string);
    /**
     * Gets or sets the Id of the action that this action will be inserted before.
     * Subsequent actions will continue to insert before the target action, following
     * the original action that set this property. If using this property do not use the
     * AfterId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get beforeId(): string;
    set beforeId(v: string);
    /**
     * Gets or sets the Id of the action that this action will be inserted after.
     * Subsequent actions will continue to insert after the target action, following
     * the original action that set this property. If using this property do not use the
     * BeforeId property.
     * This property is only relevant during initial insertion into the toolbar. Changing the property
     * after the action has already been added will have no effect.
    */
    get afterId(): string;
    set afterId(v: string);
    /**
     * Sets to the Id of the action that this action will overlay itself on to.
     * The purpose of this property is to allow users to override actions automatically generated by
     * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
     * with them that can be used for the OverlayId.
     * These overlaid actions will
     * not
     * replace the generated one but will have its properties
     * overlaid onto the generated actions.
    */
    get overlayId(): string;
    set overlayId(v: string);
    get commandId(): string;
    set commandId(v: string);
    get commandArgument(): string;
    set commandArgument(v: string);
    get commandArgumentValue(): any;
    set commandArgumentValue(v: any);
    get paddingLeft(): number;
    set paddingLeft(v: number);
    get actualPaddingLeft(): number;
    set actualPaddingLeft(v: number);
    get paddingRight(): number;
    set paddingRight(v: number);
    get actualPaddingRight(): number;
    set actualPaddingRight(v: number);
    get paddingBottom(): number;
    set paddingBottom(v: number);
    get actualPaddingBottom(): number;
    set actualPaddingBottom(v: number);
    get paddingTop(): number;
    set paddingTop(v: number);
    get actualPaddingTop(): number;
    set actualPaddingTop(v: number);
    /**
     * Gets the actual hover background used by the tool action.
    */
    get actualHoverBackground(): string;
    set actualHoverBackground(v: string);
    /**
     * Gets or sets the hover background for this tool action.
    */
    get hoverBackground(): string;
    set hoverBackground(v: string);
    /**
     * Gets the actual background used by the tool action.
    */
    get actualBackground(): string;
    set actualBackground(v: string);
    /**
     * Gets or sets the background color of the tool action.
    */
    get background(): string;
    set background(v: string);
    /**
     * Gets the actual label text color.
    */
    get actualTextColor(): string;
    set actualTextColor(v: string);
    /**
     * Gets or sets the label text color.
    */
    get textColor(): string;
    set textColor(v: string);
    /**
     * Gets the actual label text color when the tool is disabled.
    */
    get actualDisabledTextColor(): string;
    set actualDisabledTextColor(v: string);
    /**
     * Gets or sets the label text color when the tool is disabled.
    */
    get disabledTextColor(): string;
    set disabledTextColor(v: string);
    /**
     * Gets the actual font used.
    */
    get actualTextStyle(): string;
    set actualTextStyle(v: string);
    /**
     * Gets or sets the font to use.
    */
    get textStyle(): string;
    set textStyle(v: string);
    get isDisabled(): boolean;
    set isDisabled(v: boolean);
    get name(): string;
    set name(v: string);
    /**
     * Gets or sets the title of the tool.
    */
    get title(): string;
    set title(v: string);
    /**
     * Gets or sets the subtitle of the tool.
    */
    get subtitle(): string;
    set subtitle(v: string);
    get iconName(): string;
    set iconName(v: string);
    get iconCollectionName(): string;
    set iconCollectionName(v: string);
    /**
     * Gets or sets the icon viewbox left.
    */
    get iconViewBoxLeft(): number;
    set iconViewBoxLeft(v: number);
    /**
     * Gets or sets the icon viewbox top.
    */
    get iconViewBoxTop(): number;
    set iconViewBoxTop(v: number);
    /**
     * Gets or sets the icon viewbox width.
    */
    get iconViewBoxWidth(): number;
    set iconViewBoxWidth(v: number);
    /**
     * Gets or sets the icon viewbox height.
    */
    get iconViewBoxHeight(): number;
    set iconViewBoxHeight(v: number);
    /**
     * Gets or sets the icon fill color.
    */
    get iconFill(): string;
    set iconFill(v: string);
    /**
     * Gets the actual icon fill color.
    */
    get actualIconFill(): string;
    set actualIconFill(v: string);
    /**
     * Gets or sets the icon stroke color.
    */
    get iconStroke(): string;
    set iconStroke(v: string);
    /**
     * Gets the actual icon stroke color.
    */
    get actualIconStroke(): string;
    set actualIconStroke(v: string);
    /**
     * Gets or sets the tool action visibility.
    */
    get visibility(): Visibility;
    set visibility(v: Visibility);
    findByName(name: string): any;
    protected __p: string;
    protected _hasUserValues: Set<string>;
    protected get hasUserValues(): Set<string>;
    protected __m(propertyName: string): void;
    protected _stylingContainer: any;
    protected _stylingParent: any;
    protected _inStyling: boolean;
    protected _styling(container: any, component: any, parent?: any): void;
    cleanUp(): void;
    fromInfo(info: ToolActionInfo): void;
    openSubMenu(): void;
    closeSubmenu(): void;
    private _onCommand;
    private _onCommand_wrapped;
    get onCommand(): (s: IgcToolActionComponent, e: IgcToolCommandEventArgs) => void;
    set onCommand(ev: (s: IgcToolActionComponent, e: IgcToolCommandEventArgs) => void);
    private _performed;
    private _performed_wrapped;
    get performed(): (s: IgcToolActionComponent, e: IgcToolActionPerformedEventArgs) => void;
    set performed(ev: (s: IgcToolActionComponent, e: IgcToolActionPerformedEventArgs) => void);
}
